/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by client-gen. DO NOT EDIT.

package v2alpha1

import (
	"context"
	"time"

	v2alpha1 "github.com/kyverno/kyverno/api/kyverno/v2alpha1"
	scheme "github.com/kyverno/kyverno/pkg/client/clientset/versioned/scheme"
	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// PolicyExceptionsGetter has a method to return a PolicyExceptionInterface.
// A group's client should implement this interface.
type PolicyExceptionsGetter interface {
	PolicyExceptions(namespace string) PolicyExceptionInterface
}

// PolicyExceptionInterface has methods to work with PolicyException resources.
type PolicyExceptionInterface interface {
	Create(ctx context.Context, policyException *v2alpha1.PolicyException, opts v1.CreateOptions) (*v2alpha1.PolicyException, error)
	Update(ctx context.Context, policyException *v2alpha1.PolicyException, opts v1.UpdateOptions) (*v2alpha1.PolicyException, error)
	Delete(ctx context.Context, name string, opts v1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error
	Get(ctx context.Context, name string, opts v1.GetOptions) (*v2alpha1.PolicyException, error)
	List(ctx context.Context, opts v1.ListOptions) (*v2alpha1.PolicyExceptionList, error)
	Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v2alpha1.PolicyException, err error)
	PolicyExceptionExpansion
}

// policyExceptions implements PolicyExceptionInterface
type policyExceptions struct {
	client rest.Interface
	ns     string
}

// newPolicyExceptions returns a PolicyExceptions
func newPolicyExceptions(c *KyvernoV2alpha1Client, namespace string) *policyExceptions {
	return &policyExceptions{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the policyException, and returns the corresponding policyException object, and an error if there is any.
func (c *policyExceptions) Get(ctx context.Context, name string, options v1.GetOptions) (result *v2alpha1.PolicyException, err error) {
	result = &v2alpha1.PolicyException{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("policyexceptions").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of PolicyExceptions that match those selectors.
func (c *policyExceptions) List(ctx context.Context, opts v1.ListOptions) (result *v2alpha1.PolicyExceptionList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v2alpha1.PolicyExceptionList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("policyexceptions").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do(ctx).
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested policyExceptions.
func (c *policyExceptions) Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("policyexceptions").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a policyException and creates it.  Returns the server's representation of the policyException, and an error, if there is any.
func (c *policyExceptions) Create(ctx context.Context, policyException *v2alpha1.PolicyException, opts v1.CreateOptions) (result *v2alpha1.PolicyException, err error) {
	result = &v2alpha1.PolicyException{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("policyexceptions").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(policyException).
		Do(ctx).
		Into(result)
	return
}

// Update takes the representation of a policyException and updates it. Returns the server's representation of the policyException, and an error, if there is any.
func (c *policyExceptions) Update(ctx context.Context, policyException *v2alpha1.PolicyException, opts v1.UpdateOptions) (result *v2alpha1.PolicyException, err error) {
	result = &v2alpha1.PolicyException{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("policyexceptions").
		Name(policyException.Name).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(policyException).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the policyException and deletes it. Returns an error if one occurs.
func (c *policyExceptions) Delete(ctx context.Context, name string, opts v1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("policyexceptions").
		Name(name).
		Body(&opts).
		Do(ctx).
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *policyExceptions) DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Namespace(c.ns).
		Resource("policyexceptions").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched policyException.
func (c *policyExceptions) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v2alpha1.PolicyException, err error) {
	result = &v2alpha1.PolicyException{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("policyexceptions").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}
